/**
 * Custom UI-Action 'APIcall'.
 *
 * Description:
 * [Executes a REST API call to set PLC symbol on data layer]
 * lib-axios package must be installed
 * see https://axios-http.com/ how to use axios package.
 */
(function () {
    'use strict';
    var actions = shmi.pkg("visuals.session.userActions"); //get reference to userActions object

    /**
     * UI-Action 'APIcall' implementation
     *
     * @params {any[]} parameters  configured ui-action parameters
     * @param[0] {string} var name ip address
     * @param[1] {string} var name user
     * @param[2] {string} var name password
     * 
     */
    actions["APIcall"] = async function (parameters) {
        let sReq = "";
        let sResult = "";
        let bError = false;
        const im = shmi.requires("visuals.session.ItemManager");
         

        // read entered data (total 3 parameters)
        let sIP = im.readValue(parameters[0]); 
        var sUser = im.readValue(parameters[1]);
        var sPwd = im.readValue(parameters[2]);

        // other variables
        var sHost = `https://${sIP}/`;

        // function is active - show progress bar
        im.writeValue("bREST_API_call_Active", 1);

        sResult = `Host     = "${sHost}"\nUser     = "${sUser}"\nPassword = "****"\n\n`;
        // check if all data are provided
        if (sIP === "") {
            sResult += "Fill in IP address\n";
            bError = true;
        }
        if (sUser === "") {
            sResult += "Fill in user name\n";
            bError = true;
        }
        if (sPwd === "") {
            sResult += "Fill in password\n";
            bError = true;
        }

        if (bError === false) {
            // login to identify me 
            try {
                sReq = sHost + 'identity-manager/api/v1.0/auth/token';
                var response = await axios.post(sReq, { name: sUser, password: sPwd });
                if (response === null) {
                    sResult += `AXIOS call failed, check if package-axios is installed.`;
                    bError = true;
                }
            }
            catch (err) {
                sResult += `AXIOS unable to post request: ${err}\n${sReq}\nPlease check ip, user and password`;
                bError = true;
            }

            if (bError === false) {
                const token = response.data.access_token;
                const config = { headers: { authorization: 'Bearer ' + token } };

                // ***** list installed packages (=apps) *****
                try {
                    sReq = sHost + `package-manager/api/v1/packages`;
                    sResult += `\n\nRequest: ${sReq}\n`;
                    sResult += `Status: ${response.status} ${response.statusText}\n`;
                    response = await axios.get(sReq, config);
                    sResult += " Apps:\n";
                    for (let i = 0; i < response.data.length; i++) {
                        let sApp = `${response.data[i].name} ${response.data[i].release.version} ${response.data[i].appType}`;
                        sResult += `${i}:${sApp}\n`;
                    }
                }
                catch (err) {
                    sResult += `Put error: ${err}`;
                    bError = true;
                }
                // ***************  list axes *********************************
                let data = {};
                let arGetCmds = ["system/api/v1/info", "automation/api/v2/nodes/motion/axs?type=browse"];
                for (let i = 0; i < arGetCmds.length; i++) {
                    if (bError === false) {
                        // example: read system info of ctrlX
                        try {
                            sReq = sHost + arGetCmds[i];
                            sResult += `\n\nRequest: ${sReq}\n`;
                            response = await axios.get(sReq, config);
                            sResult += `Status: ${response.status} ${response.statusText}\n`;
                            if (i === 1) {
                                sResult += " Axes: "
                                for (let j = 0; j < response.data.value.length; j++) {
                                    sResult += `${j}:${response.data.value[j]}, `;
                                }
                            } else {
                                for (let sKey in response.data) {
                                    sResult += `  ${sKey} = ${response.data[sKey]}\n`;
                                }
                                //sResult += JSON.stringify(response.data, null, 2);
                            }
                        }
                        catch (err) {
                            sResult += `Put error: ${err}`;
                            bError = true;
                        }
                    }
                }

            
                // ***** dir listing of DefaultSolution: configurations/appdata/script *****
                try {
                    const sSolutionName = "DefaultSolution";
                    const sDir = "configurations/appdata/script";
                    sReq = sHost + `solutions/api/v1/solutions/${sSolutionName}/content?dir=${sDir}`;
                    sResult += `\n\nRequest: ${sReq}\n`;
                    sResult += `Status: ${response.status} ${response.statusText}\n`;
                    response = await axios.get(sReq, config);
                    sResult += " Files: "
                    for (let i = 0; i < response.data.files.length; i++) {
                        let sFile = response.data.files[i].name.replace(/"/g, '');
                        sResult += `${i}:${sFile}, `;
                    }
                }
                catch (err) {
                    sResult += `Put error: ${err}`;
                    bError = true;
                }
            
        
                // ***** list sensors *****
                try {
                    sReq = sHost + "system/api/v1/sensors";
                    sResult += `\n\nRequest: ${sReq}\n`;
                    response = await axios.get(sReq, config);
                    sResult += `Status: ${response.status} ${response.statusText}\n`;
                    
                    /*

                        let sApp = `${response.data[i].name} ${response.data[i].release.version} ${response.data[i].appType}`;
                        sResult += `${i}:${sApp}\n`;

                    */
                    sResult += `Status 2: ${response.data.length}\n`;
                    for (let j = 0; j < response.data.length; j++) 
                    {
                        sResult += `id= ${j}:${response.data[j].id}  , value= ${response.data[j].value}  , title= ${response.data[j].title}\n`;
                    }
                            
                }
                catch (err) {
                    sResult += `Put error: ${err}`;
                    bError = true;
                }
        }        
    }    
        // write interface (string) value
        im.writeValue("SString", sResult);

        // function is finished - hide progress bar
        im.writeValue("bREST_API_call_Active", 0);
    };
}());