/**
 * Custom UI-Action 'APIcall_Backup'.
 *
 * Description:
 * [Executes a REST API call to set PLC symbol on data layer]
 * lib-axios package must be installed
 * see https://axios-http.com/ how to use axios package.
 */
(function() {
    var actions = shmi.pkg("visuals.session.userActions"); //get reference to userActions object

    /**
     * UI-Action 'APIcall_Backup' implementation
     *
     * @params {any[]} parameters  configured ui-action parameters
     * @param[0] {string} var name ip address
     * @param[1] {string} var name user
     * @param[2] {string} var name password
     * 
     */

    actions["APIcall_Backup"] = async function(parameters) {
        
        let sReq = "";
        let sResult = "";
        let bError = false;
        const im = shmi.requires("visuals.session.ItemManager");
        
        // read entered data (total 4 parameters)
        let sIP = im.readValue(parameters[0]);
        var sUser = im.readValue(parameters[1]);
        var sPwd = im.readValue(parameters[2]);

        
        // other variables
        var sHost = `https://${sIP}/`;

        sResult = `APIcall_Backup: Host     = "${sHost}"\nUser     = "${sUser}"\nPassword = "****"\n\n`;
        // check if all data are provided
        if (sIP === "") {
            sResult += "Fill in IP address\n";
            bError = true;
        }
        if (sUser === "") {
            sResult += "Fill in user name\n";
            bError = true;
        }
        if (sPwd === "") {
            sResult += "Fill in password\n";
            bError = true;
        }

        if (bError === false) {

            // function is active - show progress bar
            im.writeValue("bREST_API_call_Active", 1);

            // login to identify me 
            try {
                sReq = sHost + 'identity-manager/api/v1.0/auth/token';
                var response = await axios.post(sReq, { name: sUser, password: sPwd });
                if (response === null) {
                    sResult += `AXIOS call failed, check if package-axios is installed.`;
                    bError = true;
                }
            }
            catch (err) {
                sResult += `AXIOS unable to post request: ${err}\n${sReq}\nPlease check ip, user and password`;
                bError = true;
            }
            
            // write interface (string) value
            im.writeValue("SString", sResult);

            if (bError === false) {

                const token = response.data.access_token;
                
                sResult += `\n\n download CORE solution (ZIP file)...\n`;  

                try {
                        sReq = sHost + 'solutions/api/v1/solutions/DefaultSolution/archive';
                        sResult += `\n\nRequest: ${sReq}\n`;
                        const config = { headers: { authorization: 'Bearer ' + token}, responseType:'arraybuffer'}; // responseType -> otherwise the ZIP-file is invalid
                    
                        // GET command
                        response = await axios.get(sReq, config);

                        sResult += `Status: ${response.status} ${response.statusText}\n`;

                        if (response.status >= 200 && response.status < 300)
                        {
                            sResult += `\n\n command was SUCCESSFUL \n`;
                        }

                        const url = window.URL.createObjectURL(new Blob([response.data]));
                        const link = document.createElement('a');
                        link.href = url;
                        link.setAttribute('download', 'ctrlX_CORE_Solution_Backup.zip');
                        document.body.appendChild(link);
                        link.click();
                    }
                    catch (err) {
                        sResult += `Put error: ${err}`;
                        bError = true;
                    }
            }
        }
        // write interface (string) value
        im.writeValue("SString", sResult);

        // function is finished - hide progress bar
        im.writeValue("bREST_API_call_Active", 0);
    };
}());