/**
 * Custom UI-Action 'APIcall_LoadConfiguration'.
 *
 * Description:
 * [Executes a REST API call to set PLC symbol on data layer]
 * lib-axios package must be installed
 * see https://axios-http.com/ how to use axios package.
 */
(function () {
    'use strict';
    var actions = shmi.pkg("visuals.session.userActions"); //get reference to userActions object

    /**
     * UI-Action 'APIcall_LoadConfiguration' implementation
     *
     * @params {any[]} parameters  configured ui-action parameters
     * @param[0] {string} var name ip address
     * @param[1] {string} var name user
     * @param[2] {string} var name password
     * @param[3] {string} var name configuration
     * 
     */
    actions["APIcall_LoadConfiguration"] = async function (parameters) {
        let sReq = "";
        let sResult = "";
        let bError = false;
        const im = shmi.requires("visuals.session.ItemManager");
        let bStatusState_SETUP = false;
        let bLoadConfiguration = false;
         

        // read entered data (total 3 parameters)
        let sIP = im.readValue(parameters[0]); 
        var sUser = im.readValue(parameters[1]);
        var sPwd = im.readValue(parameters[2]);
        var sConfiguration = im.readValue(parameters[3]);   // get name for the configuration

        // other variables
        var sHost = `https://${sIP}/`;

        // function is active - show progress bar
        im.writeValue("bREST_API_call_Active", 1);

        sResult = `Host     = "${sHost}"\nUser     = "${sUser}"\nPassword = "****"\n\n`;
        // check if all data are provided
        if (sIP === "") {
            sResult += "Fill in IP address\n";
            bError = true;
        }
        if (sUser === "") {
            sResult += "Fill in user name\n";
            bError = true;
        }
        if (sPwd === "") {
            sResult += "Fill in password\n";
            bError = true;
        }

        if (bError === false) {
            // login to identify me 
            try {
                sReq = sHost + 'identity-manager/api/v1.0/auth/token';
                var response = await axios.post(sReq, { name: sUser, password: sPwd });
                if (response === null) {
                    sResult += `AXIOS call failed, check if package-axios is installed.`;
                    bError = true;
                }
            }
            catch (err) {
                sResult += `AXIOS unable to post request: ${err}\n${sReq}\nPlease check ip, user and password`;
                bError = true;
            }

            if (bError === false) 
            {
                const token = response.data.access_token;
                const config = { headers: { authorization: 'Bearer ' + token } };
                let bConfigAvailable = false;

                // ***** list all configurations *****
                try {
                    sReq = sHost + `solutions/api/v1/solutions/DefaultSolution/configurations`;
                    sResult += `\n\nRequest: ${sReq}\n`;
                    sResult += `Status: ${response.status} ${response.statusText}\n`;
                    response = await axios.get(sReq, config);
                    sResult += "Available Configurations:\n";
                    for (let i = 0; i < response.data.length; i++) {
                        let sConfigs = `${response.data[i].name}`;
                        sResult += `${i}:${sConfigs}\n`;
                        // check if the expected configuration is available on the CORE:
                        if (sConfigs == sConfiguration && !bConfigAvailable)
                        {
                            bConfigAvailable = true;      
                        }
                    }
                    }
                    catch (err) {
                        sResult += `Put error: ${err}`;
                        bError = true;
                    }

                if (bConfigAvailable === true)
                {
                    // read system state
                    try {
                        sReq = sHost + `automation/api/v1/system/state`;
                        sResult += `\n\nRequest: ${sReq}\n`;
                        sResult += `Status: ${response.status} ${response.statusText}\n`;
                        response = await axios.get(sReq, config);
                        //sResult += "Available Configurations:\n";
                        
                            let sGetSystemState = `${response.data.state}`;
                            sResult += `Actual system state = ${sGetSystemState}\n`;
                            if (sGetSystemState === 'SETUP')
                            {
                                bStatusState_SETUP = true;
                            }
                        
                    }
                        catch (err) {
                            sResult += `Put error: ${err}`;
                            bError = true;
                        }


                    // activate system state: SETUP
                    if (bStatusState_SETUP === false)
                    {
                        try {
                            sReq = sHost + `automation/api/v1/scheduler/admin/state`;
                            sResult += `\n\nRequest: ${sReq}\n`;
                            let content = '{"state": "SETUP"}'; // JSON format
                            // PUT command
                            const response = await axios({
                                        method: "put",
                                        url: sReq,
                                        data: content,
                                        headers: {authorization: 'Bearer ' + token, "Content-Type": "application/json"},
                                    });

                            sResult += `Status REST-command (state): ${response.status} ${response.statusText}\n`;

                            if (response.status){
                                bLoadConfiguration = true;
                            }
                        }
                            catch (err) {
                                sResult += `Put error: ${err}`;
                                bError = true;
                            }
                    
                    }


                    // load a Configuration
                    if (bLoadConfiguration === true || bStatusState_SETUP === true)
                    {
                        try {
                            sReq = sHost + `solutions/api/v1/tasks`;
                            sResult += `\n\nRequest: ${sReq}\n`;
                            let content = `{"action": "load","properties": {"configurationPath": "solutions/DefaultSolution/configurations/${sConfiguration}"}}`; // JSON format
                            sResult += `\n content: ${content}\n`;

                            // POST command
                            const response = await axios({
                                        method: "post",
                                        url: sReq,
                                        data: content,
                                        headers: {authorization: 'Bearer ' + token, "Content-Type": "application/json"},
                                    });

                            sResult += `Status REST-command (load): ${response.status} ${response.statusText}\n`;

                            if (response.status >= 200 && response.status < 300)
                            {
                                sResult += `\n\n command was SUCCESSFUL \n`;
                            }
                            
                        }
                            catch (err) {
                                sResult += `Put error: ${err}`;
                                bError = true;
                            }
                    
                    }
                } // bConfigAvailable
                else
                {
                    sResult += `The expected config file is not available on the CORE. Please check the form data`;
                } // bConfigAvailable
            
            } 
    }    
        // write interface (string) value
        im.writeValue("SString", sResult);

        // function is finished - hide progress bar
        im.writeValue("bREST_API_call_Active", 0);
    };
}());