/**
 * Custom UI-Action 'APIcall_Upload'.
 *
 * Description:
 * [Executes a REST API call to set PLC symbol on data layer]
 * lib-axios package must be installed
 * see https://axios-http.com/ how to use axios package.
 */
(function() {
    var actions = shmi.pkg("visuals.session.userActions"); //get reference to userActions object

    /**
     * UI-Action 'APIcall_Upload' implementation
     *
     * @params {any[]} parameters  configured ui-action parameters
     * @param[0] {string} var name ip address
     * @param[1] {string} var name user
     * @param[2] {string} var name password
     * @param[3] {string} var name sConfiguration
     * 
     */
    actions["APIcall_Upload"] = async function(parameters) {
        
        let sReq = "";
        let sResult = "";
        let bError = false;
        let bDoAUpload = false;
        const im = shmi.requires("visuals.session.ItemManager");
        
        // read entered data (total 4 parameters)
        let sIP = im.readValue(parameters[0]);
        var sUser = im.readValue(parameters[1]);
        var sPwd = im.readValue(parameters[2]);
        var sConfiguration = im.readValue(parameters[3]);   // get name for the configuration

        // other variables
        var sHost = `https://${sIP}/`;

    
        sResult = `Host     = "${sHost}"\nUser     = "${sUser}"\nPassword = "****" \n\n`;
        // check if all data are provided
        if (sIP === "") {
            sResult += "Fill in IP address\n";
            bError = true;
        }
        if (sUser === "") {
            sResult += "Fill in user name\n";
            bError = true;
        }
        if (sPwd === "") {
            sResult += "Fill in password\n";
            bError = true;
        }

        if (bError === false) {

            // function is active - show progress bar
            im.writeValue("bREST_API_call_Active", 1);

            // login to identify me 
            try {
                sReq = sHost + 'identity-manager/api/v1.0/auth/token';
                var response = await axios.post(sReq, { name: sUser, password: sPwd });
                if (response === null) {
                    sResult += `AXIOS call failed, check if package-axios is installed.`;
                    bError = true;
                }
            }
            catch (err) {
                sResult += `AXIOS unable to post request: ${err}\n${sReq}\nPlease check ip, user and password`;
                bError = true;
            }

            if (bError === false) {

                const token = response.data.access_token;
                
            
                sResult += `\n\n read files inside a directory...\n`; 

                    // ***** read files inside a directory *****
                    try 
                    {
                        //requiring path and fs modules
                        const path = require('path');
                        // TEST if running on CORE: const path = shmi.requires('path');
                        const fs = require('fs');
                        // TEST if running on CORE: const fs = shmi.requires('fs');
                        //joining path of directory - the directory was expanded with a folder COREarchive
                        const directoryPath = path.join(__dirname, 'COREarchive');
                        // just debugging
                        sResult += `DIRECTORY: ${directoryPath } \n`;
                        

                        //--- start of... read files inside the directory ------------------------------
                        //passsing directoryPath and callback function
                        const files = fs.readdirSync(directoryPath)
                        for (const file of files) {
                            // print every file name found in the directory
                            sResult += `files read: ${file} \n`;
                            if (file === 'Config_Upload.zip')
                            {
                                bDoAUpload = true;
                                sResult += `perform a upload! \n`;
                            }
                        }
                        //--- end of... read files inside the directory ---------------------------------

                        
                        // ***** Upload a ZIP file to the CORE. The ZIP file is located inside the WebIQ workspace/project *****
                        if (bDoAUpload === true)
                        {
                            sReq = sHost + `solutions/api/v1/solutions/DefaultSolution/configurations/${sConfiguration}/archive`;
                            sResult += `\n Request: ${sReq}\n`;
                            sResult += `START: function readFileSync() \n`;
                            const content = fs.readFileSync(directoryPath + '\\Config_Upload.zip');
                            // print sResult += `readFileSync: \n ${content} \n`; // do not print a big zip-file.
                            
                            // upload a ZIP file
                            const response = await axios({
                                    method: "put",
                                    url: sReq,
                                    data: content,
                                    headers: {authorization: 'Bearer ' + token, "Content-Type": "application/zip"},
                                });

                            sResult += `Status PUT command: ${response.status} ${response.statusText}\n`;

                            if (response.status >= 200 && response.status < 300)
                            {
                                sResult += `\n\n command was SUCCESSFUL \n`;
                            }
                        }
                              
                    }
                        catch (err) {
                            sResult += `Put error: ${err}`;
                            bError = true;
                        }
                
            }
    }
        // write interface (string) value
        im.writeValue("SString", sResult);

        // function is finished - hide progress bar
        im.writeValue("bREST_API_call_Active", 0);
    };
}());